import logging
from .Steam import Steam
from .... import REDIRECT_URI_NAME
from django.views.generic import View
from django.shortcuts import redirect
from .models import Steam as SteamModel
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required


@method_decorator(decorator=login_required, name='dispatch')
class Authenticate(View):
    def get(self, request):
        steam = Steam()
        return_url = steam.build_url(request, '__account_steam_authenticated')
        steam_login_url = steam.login_url(request, return_url)
        return redirect(steam_login_url)


@method_decorator(decorator=login_required, name='dispatch')
class Authenticated(View):

    def get(self, request):
        steam = Steam()
        payload = steam.verify_login(request)
        self.save_connection(request, payload)
        return redirect(REDIRECT_URI_NAME)

    def save_connection(self, request, payload):
        try:
            if payload.get('steamid'):
                data = {
                    'user': request.user,
                    'steam_id': payload.get('steamid'),
                }
                SteamModel.objects.update_or_create(user=request.user, defaults=data)
        except Exception as e:
            logging.error(str(e))


@method_decorator(decorator=login_required, name='dispatch')
class UnAuthenticate(View):
    def get(self, request):
        steam = Steam()
        steam.revoke(request.user)
        return redirect(REDIRECT_URI_NAME)



from ..base_models import *
from ..services.connections.google.models import *
from ..services.connections.steam.models import *
from ..services.connections.twitch.models import *
from ..services.connections.discord.models import *



import re
import logging
import http.client
from urllib.parse import urlencode
from django.shortcuts import reverse
from .models import Steam as SteamModel
from django.utils.translation import get_language
from django.contrib.sites.shortcuts import get_current_site


class Steam:
    def get_domain(self, request):
        current_site = get_current_site(request)
        return f'{request.scheme}://{current_site}'

    def build_url(self, request, name):
        domain = self.get_domain(request)
        redirect_endpoint = reverse(name)
        if get_language():
            redirect_endpoint = redirect_endpoint.replace(f'/{get_language()}', '', 1)
        return f'{domain}{redirect_endpoint}'

    def login_url(self, request, return_url):
        domain = self.get_domain(request)
        login_url_params = {
            'openid.ns': 'http://specs.openid.net/auth/2.0',
            'openid.mode': 'checkid_setup',
            'openid.return_to': return_url,
            'openid.realm': domain,
            'openid.identity': 'http://specs.openid.net/auth/2.0/identifier_select',
            'openid.claimed_id': 'http://specs.openid.net/auth/2.0/identifier_select',
        }
        encoded_param = urlencode(login_url_params)
        steam_login_url = f'https://steamcommunity.com/openid/login?{encoded_param}'
        return steam_login_url

    def verify_login(self, request):
        response = {}
        try:
            open_id_param = {
                'openid.assoc_handle': request.GET.get('openid.assoc_handle', None),
                'openid.signed': request.GET.get('openid.signed', None),
                'openid.sig': request.GET.get('openid.sig', None),
                'openid.ns': request.GET.get('openid.ns', None),
                'openid.op_endpoint': request.GET.get('openid.op_endpoint', None),
                'openid.claimed_id': request.GET.get('openid.claimed_id', None),
                'openid.identity': request.GET.get('openid.identity', None),
                'openid.return_to': request.GET.get('openid.return_to', None),
                'openid.response_nonce': request.GET.get('openid.response_nonce', None),
                'openid.mode': 'check_authentication',
            }
            payload = urlencode(open_id_param)
            headers = {
                'Accept-language': ' en',
                'Content-type': 'application/x-www-form-urlencoded'
            }

            # verify login
            conn = http.client.HTTPSConnection('steamcommunity.com')
            conn.request('POST', '/openid/login', payload, headers)
            res = conn.getresponse()
            data = res.read()
            result = data.decode("utf-8")
            if re.search(r"is_valid\s*:\s*true", result, re.IGNORECASE):
                match = re.match(r"^https://steamcommunity.com/openid/id/(\d{17,50})",
                                 request.GET.get('openid.claimed_id'))
                steamid = match.group(1)  # Extract the Steam ID from the match object
                response.update({'steamid': steamid})
        except Exception as e:
            logging.error(str(e))
        return response

    def revoke(self, user):
        connection = SteamModel.objects.filter(user=user).first()
        if connection:
            connection.delete()



from .Google import Google


def unlink(user):
    revoke_token(user)


def refresh_token(user):
    google = Google()
    google.refresh_token(user)


def revoke_token(user):
    google = Google()
    google.revoke_token(user)



from django.conf import settings
from .apps import DjAllAuthConfig

config = DjAllAuthConfig
base_template = config.name

REDIRECT_URI_NAME = '/'
if settings.DJ_ALL_AUTH.get('CONNECTIONS'):
    __connection = settings.DJ_ALL_AUTH.get('CONNECTIONS')
    if __connection.get('REDIRECT_URI_NAME') is not None:
        REDIRECT_URI_NAME = __connection.get('REDIRECT_URI_NAME')



from .Discord import Discord


def unlink(user):
    revoke_token(user)


def refresh_token(user):
    discord = Discord()
    discord.refresh_token(user)


def revoke_token(user):
    discord = Discord()
    discord.revoke_token(user)



from .Twitch import Twitch


def unlink(user):
    revoke_token(user)


def refresh_token(user):
    twitch = Twitch()
    twitch.refresh_token(user)


def revoke_token(user):
    twitch = Twitch()
    twitch.revoke_token(user)



from django.urls import path, include

# connection service urls
from .services.connections.steam.urls import steam_urlpatterns
from .services.connections.google.urls import google_urlpatterns
from .services.connections.twitch.urls import twitch_urlpatterns
from .services.connections.discord.urls import discord_urlpatterns

urlpatterns = [
    # connection services
    path('steam/', include(steam_urlpatterns)),
    path('twitch/', include(twitch_urlpatterns)),
    path('google/', include(google_urlpatterns)),
    path('discord/', include(discord_urlpatterns)),
]



from django.contrib import admin
from .models import Google, Steam, Twitch, Discord


class DefaultPermission(admin.ModelAdmin):
    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class GoogleAdmin(DefaultPermission):
    search_fields = ('user__username', 'token_type', 'access_token', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'token_type', 'access_token', 'expires_in', 'date')


class SteamAdmin(DefaultPermission):
    search_fields = ('user__username', 'steam_id', 'date')
    list_display = ('user', 'steam_id', 'date')


class TwitchAdmin(DefaultPermission):
    search_fields = ('user__username', 'token_type', 'access_token', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'token_type', 'access_token', 'expires_in', 'date')


class DiscordAdmin(DefaultPermission):
    search_fields = ('user__username', 'identifier', 'expires_in', 'date')
    list_filter = ('token_type',)
    list_display = ('user', 'identifier', 'expires_in', 'date')


admin.site.register(Google, GoogleAdmin)
admin.site.register(Steam, SteamAdmin)
admin.site.register(Twitch, TwitchAdmin)
admin.site.register(Discord, DiscordAdmin)
